import logging
import time

from rest_framework.utils.serializer_helpers import ReturnDict

logger = logging.getLogger(__name__)


class RequestLoggingMiddleware:
    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        self.log_response(request, response)
        return response

    def process_response(self, request, response):
        self.log_response(request, response)
        return response

    def log_response(self, request, response):
        log_entry_dict = {"http.status_code": response.status_code}

        if hasattr(request, "request_start_time"):
            duration_seconds = getattr(response, "response_start_time", time.time()) - request.request_start_time
            log_entry_dict["duration"] = duration_seconds * 1000000000.0

        if 400 <= response.status_code < 500:
            log_entry_dict["error.kind"] = response.status_code
            log_entry_dict["error.message"] = response.reason_phrase
            if hasattr(response, "data") and isinstance(response.data, (list, dict, ReturnDict)):
                log_entry_dict["error.stack"] = response.data
            logger.warning(
                f"HTTP {response.status_code} {response.reason_phrase}",
                extra=log_entry_dict,
            )
        elif 500 <= response.status_code < 600:
            log_entry_dict["error.kind"] = response.status_code
            log_entry_dict["error.message"] = response.reason_phrase
            logger.error(
                f"HTTP {response.status_code} {response.reason_phrase}",
                extra=log_entry_dict,
            )
        else:
            logger.info(
                f"HTTP {response.status_code} {response.reason_phrase}",
                extra=log_entry_dict,
            )



import datetime
import re
import traceback
import typing
from logging import LogRecord

try:
    import zoneinfo
except ImportError:
    from backports import zoneinfo

import json_log_formatter
from django.conf import settings
from django.core.exceptions import DisallowedHost
from django.http.request import split_domain_port, HttpRequest
from django.urls import resolve, NoReverseMatch, Resolver404
from rest_framework.compat import unicode_http_header

import django_datadog_logger.celery
import django_datadog_logger.wsgi
from django_datadog_logger.celery import get_task_name, get_celery_request
from django_datadog_logger.encoders import SafeJsonEncoder
from django_datadog_logger.recursion import not_recursive

# those fields are excluded from extra dict
# and remains acceptable in record

EXCLUDE_FROM_EXTRA_ATTRS = {
    "user",
    "auth",
    "username",
    "request_id",
    "client_ip",
    "request",
    "celery_request",
    "wsgi_request",
    "params",
    "sql",
}


def get_client_ip(request):
    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
    if x_forwarded_for:
        return x_forwarded_for.split(",")[0] or None
    else:
        return request.META.get("REMOTE_ADDR") or None


def determine_version(request):
    if hasattr(request, "version"):
        if request.version is not None:
            return str(request.version)
    elif hasattr(request, "accepted_types"):
        for t in request.accepted_types:
            if t.params.get("version") is not None:
                return unicode_http_header(t.params.get("version"))
    return None


@not_recursive
def get_wsgi_request_auth(wsgi_request):
    try:
        if getattr(wsgi_request, "auth", None) is not None and isinstance(wsgi_request.auth, dict):
            return wsgi_request.auth
    except Exception:  # NOQA
        return None


@not_recursive
def get_wsgi_request_user(wsgi_request):
    if getattr(wsgi_request, "user", None) is not None:
        if getattr(wsgi_request.user, "is_authenticated", False):
            return wsgi_request.user


class DataDogJSONFormatter(json_log_formatter.JSONFormatter):
    def json_record(self, message: str, extra: typing.Dict, record: LogRecord) -> typing.Dict:
        log_entry_dict = {
            "message": message,
            "logger.name": record.name,
            "logger.thread_name": record.threadName,
            "logger.method_name": record.funcName,
            "date": (
                datetime.datetime.utcfromtimestamp(record.created).replace(tzinfo=zoneinfo.ZoneInfo("UTC")).isoformat()
            ),
            "status": record.levelname,
        }

        # Add special `dd.` log record attributes added by `ddtrace` library
        # For example: dd.trace_id, dd.span_id, dd.service, dd.env, dd.version, etc
        log_entry_dict.update(self.get_datadog_attributes(record))

        wsgi_request = self.get_wsgi_request()
        if wsgi_request is not None:
            log_entry_dict["network.client.ip"] = get_client_ip(wsgi_request)

            try:
                domain, port = split_domain_port(wsgi_request.get_host())
            except DisallowedHost:
                domain, port = None, None

            try:
                resolver_match = resolve(wsgi_request.path)
            except (NoReverseMatch, Resolver404):
                resolver_match = None

            log_entry_dict["http.url"] = wsgi_request.get_full_path()
            log_entry_dict["http.url_details.host"] = domain
            log_entry_dict["http.url_details.port"] = int(port) if port else None
            log_entry_dict["http.url_details.path"] = wsgi_request.path_info
            log_entry_dict["http.url_details.queryString"] = wsgi_request.GET.dict()
            log_entry_dict["http.url_details.scheme"] = wsgi_request.scheme
            log_entry_dict["http.url_details.view_name"] = resolver_match.view_name if resolver_match else None
            log_entry_dict["http.method"] = wsgi_request.method
            log_entry_dict["http.accept"] = wsgi_request.META.get("HTTP_ACCEPT")
            log_entry_dict["http.referer"] = wsgi_request.META.get("HTTP_REFERER")
            log_entry_dict["http.useragent"] = wsgi_request.META.get("HTTP_USER_AGENT")
            log_entry_dict["http.request_version"] = determine_version(wsgi_request)

            if hasattr(wsgi_request, "request_id"):
                log_entry_dict["http.request_id"] = wsgi_request.request_id

            try:
                auth = get_wsgi_request_auth(wsgi_request)
            except:  # NOQA: we cannot crash the log formatter under any circumstances
                auth = None

            if auth:
                if "sid" in auth:
                    log_entry_dict["usr.session_id"] = auth["sid"]
                if "cid" in auth:
                    log_entry_dict["usr.client_id"] = auth["cid"]

            try:
                user = get_wsgi_request_user(wsgi_request)
            except:  # NOQA: we cannot crash the log formatter under any circumstances
                user = None

            if user:
                log_entry_dict["usr.id"] = getattr(user, "pk", None)
                log_entry_dict["usr.name"] = getattr(user, getattr(user, "USERNAME_FIELD", "username"), None)
                log_entry_dict["usr.email"] = getattr(user, "email", None)

            if getattr(wsgi_request, "session", None) is not None and getattr(wsgi_request.session, "session_key"):
                log_entry_dict["usr.session_key"] = wsgi_request.session.session_key

        if record.exc_info:
            if hasattr(record, "status_code"):
                log_entry_dict["error.kind"] = record.status_code
                log_entry_dict["error.message"] = record.msg
            elif record.exc_info[0] is not None:
                log_entry_dict["error.kind"] = record.exc_info[0].__name__
                for msg in traceback.format_exception_only(record.exc_info[0], record.exc_info[1]):
                    log_entry_dict["error.message"] = msg.strip()
            log_entry_dict["error.stack"] = self.formatException(record.exc_info)

        if hasattr(record, "duration"):
            log_entry_dict["duration"] = record.duration

        if hasattr(record, "sql"):
            log_entry_dict["db.statement"] = record.sql

        celery_request = get_celery_request()
        if celery_request is not None:
            log_entry_dict["celery.request_id"] = celery_request.id
            log_entry_dict["celery.task_name"] = get_task_name(celery_request)
        elif record.name in {"celery.app.trace", "celery.worker.strategy"} and "data" in extra:
            if "id" in extra["data"]:
                log_entry_dict["celery.request_id"] = extra["data"]["id"]
            if "name" in extra["data"]:
                log_entry_dict["celery.task_name"] = extra["data"]["name"]
            if "runtime" in extra["data"]:
                log_entry_dict["duration"] = extra["data"]["runtime"] * 1000000000

        if hasattr(settings, "DJANGO_DATADOG_LOGGER_EXTRA_INCLUDE"):
            if re.match(getattr(settings, "DJANGO_DATADOG_LOGGER_EXTRA_INCLUDE"), record.name):
                log_entry_dict.update(extra)

        return log_entry_dict

    def get_datadog_attributes(self, record: LogRecord) -> typing.Dict:
        """Helper to extract dd.* attributes from the log record."""
        return {attr_name: record.__dict__[attr_name] for attr_name in record.__dict__ if attr_name.startswith("dd.")}

    def get_wsgi_request(self) -> typing.Optional[HttpRequest]:
        return django_datadog_logger.wsgi.get_wsgi_request()

    def to_json(self, record: typing.Dict) -> str:
        """Converts record dict to a JSON string."""
        return self.json_lib.dumps(record, cls=SafeJsonEncoder)

    def extra_from_record(self, record: LogRecord) -> typing.Dict:
        """Returns `extra` dict you passed to logger.

        The `extra` keyword argument is used to populate the `__dict__` of
        the `LogRecord`.

        """
        return {
            attr_name: record.__dict__[attr_name]
            for attr_name in record.__dict__
            if attr_name not in json_log_formatter.BUILTIN_ATTRS.union(EXCLUDE_FROM_EXTRA_ATTRS)
        }



from asgiref.local import Local  # NOQA

local = Local()


def get_wsgi_request():
    try:
        return local.request
    except AttributeError:
        return None


__all__ = ["local", "get_wsgi_request"]



import re
import time
import uuid

from django_datadog_logger.wsgi import local


def generate_request_id():
    return str(uuid.uuid4())


def get_or_create_request_id(request):
    request_id = request.META.get("HTTP_X_REQUEST_ID")
    if request_id and re.match("^[a-zA-Z0-9+/=-]{20,200}$", request_id):
        return request_id
    else:
        return generate_request_id()


class RequestIdMiddleware:
    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        request.request_id = get_or_create_request_id(request)
        request.request_start_time = time.time()
        local.request = request
        response = self.get_response(request)
        response["X-Request-ID"] = request.request_id
        if hasattr(local, "request"):
            del local.request
        return response



from functools import wraps

from asgiref.local import Local  # NOQA

local = Local()


def get_celery_request():
    try:
        return local.request
    except AttributeError:
        return None


def get_task_name(request):
    if hasattr(request, "task"):
        if isinstance(request.task, str):
            return request.task
        elif hasattr(request.task, "name"):
            return request.task.name
    return None


def store_celery_request(func):
    @wraps(func)
    def function_wrapper(*args, **kwargs):
        try:
            if args and hasattr(args[0], "request"):
                request = args[0].request
                if (type(request).__module__, type(request).__name__) == ("celery.app.task", "Context"):
                    local.request = request
            return func(*args, **kwargs)
        finally:
            if hasattr(local, "request"):
                del local.request

    return function_wrapper


__all__ = ["local", "get_celery_request", "store_celery_request"]



"""Tests for `django_datadog_logger` package."""

import logging
import unittest

from django.conf import settings

from django_datadog_logger.formatters.datadog import DataDogJSONFormatter


class DjangoDatadogLoggerTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        if not settings.configured:
            settings.configure()

    def test_format_json_accepts_a_tuple_of_nones_as_exc_info(self):
        """
        When logger is called with exc_info=True, then the exc_info
        attribute of the LogRecord is a tuple of (None, None, None).
        """
        record = logging.LogRecord("foo", logging.ERROR, "foo.py", 42, "This is an error", None, (None, None, None))
        formatter = DataDogJSONFormatter()
        json_record = formatter.json_record("Foo", {}, record)

        self.assertEqual(json_record.get("error.kind"), None)



import copy
import logging

from django.db import transaction
from django.db.models import Manager, QuerySet
from rest_framework.fields import ModelField

logger = logging.getLogger(__name__)


class ActionLogMixin:
    @staticmethod
    def get_create_log_entity(serializer, instance, action):
        validated_data = copy.copy(serializer.validated_data)
        log_entity = {"pk": instance.pk}
        ModelClass = serializer.Meta.model

        for field_name, field in serializer.fields.items():
            if field.source == "*":
                value = instance
            elif field.source in validated_data:
                value = validated_data.pop(field.source, None)
            else:
                value = getattr(instance, field.source, None)

            if isinstance(field, ModelField):
                log_entity[field_name] = field.to_representation(instance)
            elif isinstance(value, (Manager, QuerySet)):
                log_entity[field_name] = field.to_representation(value.all())
            elif callable(value):
                log_entity[field_name] = field.to_representation(value())
            elif value is not None:
                log_entity[field_name] = field.to_representation(value)
            else:
                log_entity[field_name] = None

        for attr, value in validated_data.items():
            log_entity[attr] = repr(value)

        message = "{} {}".format(ModelClass.__name__, action)
        extra = {"{}.{}".format(ModelClass._meta.app_label, ModelClass.__name__): log_entity}
        return message, extra

    def perform_create(self, serializer):
        super().perform_create(serializer)
        message, extra = self.get_create_log_entity(serializer, serializer.instance, "created")
        logger.info(message, extra=extra)

    def perform_update(self, serializer):
        super().perform_update(serializer)
        message, extra = self.get_create_log_entity(serializer, serializer.instance, "updated")
        logger.info(message, extra=extra)

    def perform_destroy(self, instance):
        with transaction.atomic():
            msg = {"pk": instance.pk}
            super().perform_destroy(instance)
            logger.info(
                "{} deleted".format(type(instance).__name__),
                extra={"{}.{}".format(instance._meta.app_config.label, type(instance).__name__): msg},
            )



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.md") as history_file:
    history = history_file.read()

requirements = []
with open("requirements.txt") as requirements_file:
    for line in requirements_file.readlines():
        # skip to next iteration if comment or empty line
        if line.startswith("#") or line == "" or line.startswith("http") or line.startswith("git"):
            continue
        # add line to requirements
        requirements.append(line)

setup_requirements = []

setup(
    author="Lenno Nagel",
    author_email="lenno@namespace.ee",
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    description="Django DataDog Logger integration package.",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="django_datadog_logger",
    name="django-datadog-logger",
    packages=find_packages(include=["django_datadog_logger", "django_datadog_logger.*"]),
    setup_requires=setup_requirements,
    url="https://github.com/namespace-ee/django-datadog-logger",
    version="0.7.2",
    zip_safe=False,
)



import logging

from django.core.exceptions import PermissionDenied, BadRequest, SuspiciousOperation
from django.http import Http404
from django.http.multipartparser import MultiPartParserError

logger = logging.getLogger(__name__)


class ErrorLoggingMiddleware:
    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        return self.get_response(request)

    def process_exception(self, request, exception):
        if isinstance(
            exception,
            (PermissionDenied, Http404, MultiPartParserError, BadRequest, SuspiciousOperation),
        ):
            return
        logger.exception(exception)

    def process_response(self, request, response):
        return response



import inspect
from functools import wraps


class RecursionDetected(RuntimeError):
    """function has been detected to be recursing"""


def not_recursive(f):
    """
    raise an exception if recursive
    """

    @wraps(f)
    def wrapper(*args, **kwargs):
        for frame in inspect.stack():
            if f.__name__ == frame.function:
                raise RecursionDetected(f"function '{f.__name__}' is recursive")

        return f(*args, **kwargs)

    return wrapper



from django.core.serializers.json import DjangoJSONEncoder


class SafeJsonEncoder(DjangoJSONEncoder):
    def default(self, o):
        try:
            return super(SafeJsonEncoder, self).default(o)
        except TypeError as e:
            return str(e)
